import domina_util as util

import click

PATH_SRC='src'
PATH_BUILD='build'
PATH_DIST='dist'

URL_UPSTREAM = util.getenv(
    'DOMINA_URL_UPSTREAM',
    default='https://gitlab.com/screeps-domina/domina')

PATH_CONFIG_SAMPLE = util.getenv(
    'DOMINA_PATH_CONFIG_SAMPLE',
    default='config.sample.json')
PATH_CONFIG = util.getenv(
    'DOMINA_PATH_CONFIG',
    default='config.json')

GEN = util.getenv(
    'DOMINA_GEN',
    default=util.get_latest_generation(PATH_CONFIG)['gen'])
GENERATION = util.get_generation(PATH_CONFIG, GEN)

DEPLOY_TARGET = util.getenv(
    'DOMINA_DEPLOY_TARGET',
    default='screeps.com')

BUILD_PATHS = dict(
    path_src= util.join(PATH_SRC, GENERATION['name']),
    path_build = util.join(PATH_BUILD, GENERATION['name']),
    path_dist = util.join(PATH_DIST, GENERATION['name']),
)

@click.group()
@click.version_option()
def cli():
    '''Evolving game AI script framework for MMORTS screeps.com. Can be
    used to play screeps from a remote machine, or build CI/CD
    pipelines with comfort.

    It is presumed you want to manage your latest generation. You can
    override this behavior by setting the DOMINA_GEN environment
    variable to a desired number.

    '''
    pass

@cli.command('init')
def cmd_init():
    '''Initialize domina in current directory.'''
    branch='feature/domina-cli'
    util.download_file(f'{URL_UPSTREAM}/-/raw/{branch}/{PATH_CONFIG_SAMPLE}',
                       PATH_CONFIG_SAMPLE)

@cli.group('list')
def list_():
    '''List available objects.'''
    pass

@list_.command('generations')
def cmd_list_generations():
    '''List generations specified in configuration.'''
    for generation in util.get_generations(PATH_CONFIG):
        click.echo(f'[{generation["gen"]}] {generation["name"]}')

@list_.command('targets')
def cmd_list_targets():
    '''List targets specified in configuration.'''
    for target in util.get_names(util.get_targets(PATH_CONFIG)):
        click.echo(target)

@cli.command('configure')
@click.option('--init', '-I', is_flag=True,
              help='Always initialize with latest configuration.')
@click.option('--pass', '-p' 'pass_name',
              metavar='<pass-name>', default=False,
              help='Use pass variable as sample config.')
@click.option('--overwrite', '-f', is_flag=True,
              help='Overwrite existing configuration.')
@click.option('--edit/--no-edit', default=False,
              help='Edit configuration file.')
@click.option('--download/--skip-download' '-d', default=False,
              help='Download source files.')
def cmd_configure(init, pass_name, overwrite, edit, download):
    '''Configure this repository.'''
    if not util.file_exists(PATH_CONFIG) or overwrite:
        if pass_name:
            util.pass_to_file(PATH_CONFIG, pass_name)
        elif util.file_exists(PATH_CONFIG_SAMPLE):
            util.copy(PATH_CONFIG_SAMPLE, PATH_CONFIG)
    if edit:
        util.open_editor(PATH_CONFIG)
    if download:
        util.clone_sources(PATH_CONFIG, PATH_SRC)

@cli.command('compile')
@click.option('--flatten/--no-flatten', default=True,
              help='Flatten nested submodules', show_default=True)
def cmd_compile(flatten):
    '''Build the project.'''
    util.build_project(flatten, **BUILD_PATHS)

@cli.command('test')
@click.option('--build/--no-build', default=True,
              help='Compile source files', show_default=True)
def cmd_test(build):
    '''Test the built project.'''
    if build:
        util.build_project(distribute=False, **BUILD_PATHS)
    util.test_project(util.join(PATH_BUILD, GENERATION['name']))

@cli.command('deliver')
@click.argument('target')
@click.confirmation_option(prompt='Are you sure?')
@click.option('--dry-run', '-C', is_flag=True, default=False,
              help='Do not affect the target')
@click.option('--build/--no-build', default=True,
              help='Compile source files', show_default=True)
@click.option('--test/--skip-test', default=True,
              help='Run tests (implicit --build)', show_default=True)
def cmd_deliver(target, build, test, dry_run):
    '''Deliver the project to TARGET.'''
    target = util.get_target(PATH_CONFIG, target)
    if target is None:
        raise click.BadParameter(
            util.get_names(util.get_targets(PATH_CONFIG)),
            param=target, param_hint='TARGET: Acceptable values are')
    util.deliver_project(target, build, test, dry_run, **BUILD_PATHS)

@cli.command('deploy')
@click.confirmation_option(prompt='Are you sure?')
@click.option('--dry-run', '-C', is_flag=True, default=False,
              help='Do not affect the target')
@click.option('--build/--no-build', default=True,
              help='Compile source files', show_default=True)
@click.option('--test/--skip-test', default=True,
              help='Run tests (implicit --build)', show_default=True)
def cmd_deploy(build, test, dry_run):
    '''Deploy the project to official server.'''
    target = util.get_target(PATH_CONFIG, 'screeps.com')
    util.deliver_project(target, build, test, dry_run, **BUILD_PATHS)

@click.command('distribute')
@click.confirmation_option(prompt=f'Distribute domina-{util.read_version()}?')
@click.option('--dry-run', '-C', is_flag=True, default=False,
              help='Do not affect the target')
@click.option('--pass', '-p', 'pass_name',
              metavar='<pass-name>', default=False,
              help='Use pass variable as release token.')
def cmd_distribute(pass_name, dry_run):
    '''(dev) Build and distribute domina itself.'''
    token = util.pass_decrypt(pass_name) \
        if pass_name else None
    util.distribute_domina(PATH_DIST, token=token)

if __name__ == '__main__':
    cli.add_command(cmd_distribute)
    cli()



import click
import os
import json
import shutil

from os import getcwd as rootdir
from os import getenv
from os.path import join
from os.path import isfile as file_exists
from os.path import isdir as directory_exists
from shutil import rmtree as remove
from subprocess import run

GEN_0 = dict(gen=0, name='0')

def get_names(dicts):
    return list(map(lambda t: t['name'], dicts))

def colorline(color, status, *msg):
    return click.echo(' '.join([click.style(status, fg=color), *msg]))

def stepline(*msg):
    return colorline('green', '*', *msg)

def clickfail(msg):
    click.get_current_context().fail(msg)

def check_file_exists(path):
    if not file_exists(path):
        raise click.FileError(path, hint='No such file')

def open_editor(path):
    stepline(f'Editing {path}')
    check_file_exists(path)
    click.edit(filename=path)

def read_file(path):
    check_file_exists(path)
    with open(path, 'r') as f:
        return f.read()

def read_json(path):
    return json.loads(read_file(path))

def read_about():
    return str(read_file('ABOUT'))

def read_version():
    return str(read_file('VERSION'))

def get_targets(path_config):
    return read_json(path_config)['targets']

def get_target(path_config, name, fail=False):
    targets, target = get_targets(path_config), None
    for t in filter(lambda t: t['name'] == name, targets):
        target = t
        break
    if fail and target is None:
        raise click.BadParameter(
            get_names(get_targets(path_config)),
            param=target,
            param_hint='TARGET: Acceptable values are'
        )
    return target

def get_generations(path_config):
    return read_json(path_config)['evolution']

def get_generation(path_config, gen, fail=False):
    for generation in get_generations(path_config):
        if generation['gen'] == gen:
            return generation
    if fail:
        clickfail(f'No generation with gen: {gen}')
    return GEN_0

def get_latest_generation(path_config):
    latest, gen = None, int()
    for generation in get_generations(path_config):
        if generation['gen'] >= gen:
            latest, gen = generation, generation['gen']
    return latest if latest is not None else GEN_0

def pass_decrypt(pass_name):
    stepline(f'Decrypting <{pass_name}>')
    return str(run(f'pass show {pass_name}'.split(),
                   capture_output=True).stdout, encoding='utf-8').strip()

def pass_to_file(path, pass_name):
    decrypted_pass = pass_decrypt(pass_name)
    with open(path, 'w') as f:
        f.writelines(decrypted_pass)

def download_file(url, path):
    stepline(f'Downloading {url} to {path}')
    run(f'curl {url} -o {path}'.split())

def copy(path_orig, path_dest):
    if directory_exists(path_orig):
        stepline(f'Copying directory {path_orig} over {path_dest}')
        shutil.copytree(path_orig, path_dest)
    elif file_exists(path_orig):
        stepline(f'Copying file {path_orig} over {path_dest}')
        shutil.copy(path_orig, path_dest)
    else:
        raise click.FileError(path_orig, hint='No such file')

def clone_sources(path_config, path_src):
    if not directory_exists(path_src):
        os.makedirs(path_src)
    for gen in get_generations(path_config):
        if 'git' in gen:
            version = gen.setdefault('version', 'master')
            stepline(f'Cloning {gen["name"]} at {version}')
            run(['git', 'clone {gen["git"]}',
                 '{path_src}/{gen["name"]}',
                 '-b {version}'])

def flatten_submodules(path_root):
    pass

def run_tests(path):
    path_bin_mocha = './node_modules/mocha/bin/mocha'
    process = run(f'{path_bin_mocha} {path}', shell=True)
    return process.returncode == 0

def test_project(path_build, fail=False):
    stepline('Running test suites')
    test_ok = run_tests(join(path_build, 'test'))
    if not test_ok and fail:
        fail('Failed to pass the tests.')
    return test_ok

def build_project(flatten=True, test=False, distribute=True,
                  path_src=None, path_build='build', path_dist='dist'):
    if path_src is None or not directory_exists(path_src):
        msg = 'Source file not present'
        raise click.FileError(f'{path_src}/main/main.js', hint=msg)
    if directory_exists(path_build):
        stepline('Clearing previous build')
        remove(path_build)
    stepline('Copying source files')
    copy(path_src, path_build)
    stepline('Compiling source files')
    if flatten:
        stepline('Flattening source files')
        flatten_submodules(path_build)
    if test:
        test_project(path_build, fail=True)
    if distribute:
        if directory_exists(path_dist):
            stepline('Clearing previous artifacts')
            remove(path_dist)
        stepline('Copying artifacts')
        copy(path_build, path_dist)

def deliver_project(target, build, test, dry_run, **build_paths):
    if build or test:
        build_project(flatten=True, test=test, distribute=True,
                      **build_paths)
    if not dry_run:
        stepline(f'Delivering to {target["name"]}')

def distribute_domina(path_dist='dist', repo='pypi', token=None):
    version = read_version()
    artifacts = join(path_dist, 'domina')+'-*'
    stepline('Clearing previous artifacts')
    run(f'rm -f {artifacts}'.split())
    stepline(f'Building domina-{version}')
    run('python setup.py sdist bdist_wheel'.split())
    if token is not None:
        stepline(f'Distributing domina-{version} to pypi')
        run(['twine', 'upload', artifacts, '-u', '__token__', '-p', token,
             '-r', repo])




import setuptools
import domina_util as util

setuptools.setup(
    name='domina',
    version=util.read_file('VERSION'),
    author='hexanona',
    author_email=''.join(
        ['hexanona','+','domina','@','mailbox.org']),
    description='Game AI script framework for MMORTS screeps.com',
    long_description=util.read_file('ABOUT'),
    long_description_content_type='text/plain',
    url='https://gitlab.com/screeps-domina/domina',
    classifiers=[
        'Topic :: Games/Entertainment :: Real Time Strategy',
        'Topic :: Software Development :: Version Control',
        'Intended Audience :: Developers',
        'Environment :: Console',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: JavaScript',
        'Development Status :: 1 - Planning',
    ],
    license='GPLv3',
    py_modules=[
        'domina',
        'domina_util',
    ],
    install_requires=['Click'],
    entry_points='''
    [console_scripts]
    domina=domina:cli
    ''',
    python_requires='>=3.7'
)
